@@ -9,11 +9,12 @@ import httplib
 import urllib2
 from fnmatch import fnmatch
 from requests import Request, Session
+from requests.exceptions import ConnectionError
 from contextlib import closing
 
 from jsonpatch import make_patch
 
-from . import ias3, config, utils
+from . import s3, config, __version__
 
 
 
@@ -65,24 +66,18 @@ class Item(object):
                        otherwise use HTTP.
 
         """
-        self.identifier = identifier
         self.secure = secure
-        if secure:
-            protocol = 'https'
-        else:
-            protocol = 'http'
-        self.details_url = '{0}://archive.org/details/{1}'.format(protocol, identifier)
-        self.download_url = '{0}://archive.org/download/{1}'.format(protocol, identifier)
-        self.metadata_url = '{0}://archive.org/metadata/{1}'.format(protocol, identifier)
-        self.s3_endpoint = '{0}://s3.us.archive.org/{1}'.format(protocol, identifier)
+        protocol = 'https://' if secure else 'http://'
+        self.host = protocol + 'archive.org'
+        self.identifier = identifier
+        self.details_url = self.host + '/details/' + self.identifier
+        self.download_url = self.host + '/download/' + self.identifier
+        self.metadata_url = self.host + '/metadata/' + self.identifier
+        self.s3_endpoint = protocol + 's3.us.archive.org' + self.identifier
         self.metadata_timeout = metadata_timeout
         self.session = None
-        self.bucket = None
-        self.metadata = utils.get_item_metadata(identifier, metadata_timeout, secure)
-        if self.metadata == {}:
-            self.exists = False
-        else:
-            self.exists = True
+        self.metadata = self.get_metadata()
+        self.exists = False if self.metadata == {} else True
 
 
     # __repr__()
@@ -100,6 +95,34 @@ class Item(object):
                 'files_count={files_count!r})'.format(**item_description))
 
 
+    # get_metadata()
+    #_____________________________________________________________________________________
+    def get_metadata(self, target=None):
+        """Get an item's metadata from the `Metadata API 
+        <http://blog.archive.org/2013/07/04/metadata-api/>`__
+
+        :type identifier: str
+        :param identifier: Globally unique Archive.org identifier.
+
+        :type target: bool
+        :param target: (optional) Metadata target to retrieve.
+
+        :rtype: dict
+        :returns: Metadat API response.
+
+        """
+        if not self.session:
+            self.session = Session()
+        response = self.session.get(self.metadata_url, timeout=self.metadata_timeout)
+        if response.status_code != 200:
+            raise ConnectionError("Unable connect to Archive.org "
+                                  "({0})".format(response.status_code))
+        metadata = response.json()
+        if target:
+            metadata = metadata.get(target, {})
+        return metadata
+
+
     # files()
     #_____________________________________________________________________________________
     def files(self):
@@ -188,10 +211,11 @@ class Item(object):
         for f in files:
             fname = f.name.encode('utf-8')
             path = os.path.join(self.identifier, fname)
+            if dry_run:
+                stdout.write(f.url + '\n')
+                continue
             if verbose:
                 stdout.write('downloading: {0}\n'.format(fname))
-            if dry_run:
-                stdout.write('{0}\n'.format(f.download_url))
             elif concurrent:
                 pool.spawn(f.download, path, ignore_existing=ignore_existing)
             else:
@@ -265,8 +289,7 @@ class Item(object):
         http.send(data)
         status_code, error_message, headers = http.getreply()
         resp_file = http.getfile()
-        self.metadata = utils.get_item_metadata(self.identifier, self.metadata_timeout,
-                                                self.secure)
+        self.metadata = self.get_metadata()
         return dict(
             status_code = status_code,
             content = json.loads(resp_file.read()),
@@ -275,83 +298,83 @@ class Item(object):
 
     # upload_file()
     #_____________________________________________________________________________________
-    def upload_file(self, local_file, remote_name=None, metadata={}, headers={},
-                    queue_derive=True, ignore_bucket=False, verbose=False, debug=False):
+    def upload_file(self, body, key=None, metadata={}, headers={},
+                    access_key=None, secret_key=None, queue_derive=True, 
+                    ignore_preexisting_bucket=False, verbose=False, debug=False):
         """Upload a single file to an item. The item will be created
         if it does not exist.
 
-        :type local_file: str or file
-        :param local_file: The filepath or file-like object to be uploaded.
+        :type body: Filepath or file-like object.
+        :param body: File or data to be uploaded.
 
-        :type remote_name: str
-        :param remote_name: (optional) Sets the remote filename.
+        :type key: str
+        :param key: (optional) Remote filename.
 
         :type metadata: dict
         :param metadata: (optional) Metadata used to create a new item.
 
         :type headers: dict
-        :param headers: (optional) Add additional IA-S3 headers to
-                        request.
+        :param headers: (optional) Add additional IA-S3 headers to request.
 
         :type queue_derive: bool
         :param queue_derive: (optional) Set to False to prevent an item from
                              being derived after upload.
 
-        :type ignore_bucket: bool
-        :param ignore_bucket: (optional) Set to True to ignore and
-                              clobber existing files and metadata.
+        :type ignore_preexisting_bucket: bool
+        :param ignore_preexisting_bucket: (optional) Destroy and respecify the 
+                                          metadata for an item
+
+        :type verbose: bool
+        :param verbose: (optional) Print progress to stdout.
 
         :type debug: bool
-        :param debug: (optional) Set to True to print headers to stdout,
-                      and exit without sending the upload request.
+        :param debug: (optional) Set to True to print headers to stdout, and 
+                      exit without sending the upload request.
 
         Usage::
 
             >>> import internetarchive
             >>> item = internetarchive.Item('identifier')
             >>> item.upload_file('/path/to/image.jpg',
-            ...                  remote_name='photos/image1.jpg')
+            ...                  key='photos/image1.jpg')
             True
 
-        :rtype: bool
-        :returns: True if the request was successful and file was
-                  uploaded, False otherwise.
-
         """
-        if not self.session:
-            self.session = Session()
+        if not hasattr(body, 'read'):
+            body = open(body, 'rb')
+        if not metadata.get('scanner'):
+            scanner = 'Internet Archive Python library {0}'.format(__version__)
+            metadata['scanner'] = scanner
+        try:
+            body.seek(0, os.SEEK_END)
+            size = body.tell()
+            body.seek(0, os.SEEK_SET)
+        except IOError:
+            size = None
 
-        if not hasattr(local_file, 'read'):
-            local_file = open(local_file, 'rb')
-        if not remote_name:
-            remote_name = local_file.name.split('/')[-1]
-
-        # Attempt to add size-hint header.    
-        if not headers.get('x-archive-size-hint'):
-            try:
-                local_file.seek(0, os.SEEK_END)
-                headers['x-archive-size-hint'] = local_file.tell()
-                local_file.seek(0, os.SEEK_SET)
-            except IOError:
-                pass
-
-        # Prepare Request.
-        endpoint = 'http://s3.us.archive.org/{0}/{1}'.format(self.identifier, remote_name)
-        headers = ias3.build_headers(metadata, headers, queue_derive=queue_derive,
-                                     ignore_bucket=ignore_bucket)
-        request = Request('PUT', endpoint, headers=headers)
-        # TODO: Add support for multipart.
-        # `contextlib.closing()` is used to make StringIO work with 
-        # `with` statement.
-        with closing(local_file) as data:
-            request.data = data
-            prepped_request = request.prepare()
-            if debug:
-                return prepped_request 
-            else:
-                if verbose:
-                    stdout.write(' uploading file: {0}\n'.format(remote_name))
-                return self.session.send(prepped_request, stream=True)
+        key = body.name.split('/')[-1] if key is None else key
+        url = 'http://s3.us.archive.org/{0}/{1}'.format(self.identifier, key) 
+        headers = s3.build_headers(metadata=metadata, 
+                                   headers=headers, 
+                                   queue_derive=queue_derive,
+                                   auto_make_bucket=True,
+                                   size_hint=size,
+                                   ignore_preexisting_bucket=ignore_preexisting_bucket)
+        request = Request(
+            method='PUT',
+            url=url,
+            headers=headers,
+            data=body,
+            auth=s3.BasicAuth(access_key, secret_key),
+        )
+        
+        if debug:
+            return request
+        else:
+            if not self.session:
+                self.session = Session()
+            prepared_request = request.prepare()
+            return self.session.send(prepared_request, stream=True)
 
 
     # upload()
@@ -384,20 +407,21 @@ class Item(object):
             for path, dir, files in os.walk(directory):
                 for f in files:
                     filepath = os.path.join(path, f)
-                    remote_name = os.path.relpath(filepath, directory)
-                    yield (filepath, remote_name)
+                    key = os.path.relpath(filepath, directory)
+                    yield (filepath, key)
 
         if not isinstance(files, (list, tuple)):
             files = [files]
 
         responses = []
-        for local_file in files:
-            if isinstance(local_file, basestring) and os.path.isdir(local_file):
-                for local_file, remote_name in iter_directory(local_file):
-                    resp = self.upload_file(local_file, remote_name=remote_name, **kwargs)
+        for f in files:
+            key = None
+            if isinstance(f, basestring) and os.path.isdir(f):
+                for filepath, key in iter_directory(f):
+                    resp = self.upload_file(filepath, key=key, **kwargs)
                     responses.append(resp)
             else:
-                resp = self.upload_file(local_file, **kwargs)
+                resp = self.upload_file(f, **kwargs)
                 responses.append(resp)
         return responses
 
@@ -413,20 +437,16 @@ class File(object):
         self.external_identifier = file_dict.get('external-identifier')
         self.name = file_dict.get('name')
         self.source = file_dict.get('source')
-        self.size = file_dict.get('size')
-        self.size = file_dict.get('size')
-        if self.size is not None:
-            self.size = int(self.size)
+        self.size = int(file_dict.get('size')) if file_dict.get('size') else None
         self.format = file_dict.get('format')
         self.mtime = file_dict.get('mtime')
         self.md5  = file_dict.get('md5')
-        self.sha1 = file_dict.get('crc32')
+        self.crc32 = file_dict.get('crc32')
         self.sha1 = file_dict.get('sha1')
         self.fname = self.name.encode('utf-8')
-        self.download_url = '{0}/{1}'.format(self.item.download_url, 
-                                             urllib.quote(self.fname, safe=''))
-        if not self.item.session:
-            self.item.session = Session()
+        self.length = float(file_dict.get('length')) if file_dict.get('length') else None
+        self.url = self.item.download_url + '/' + urllib.quote(self.fname, safe='')
+        self.session = self.item.session if self.item.session else Session()
 
 
     # __repr__()
@@ -443,9 +463,7 @@ class File(object):
     #_____________________________________________________________________________________
     def download(self, file_path=None, ignore_existing=False):
         """:todo: document ``internetarchive.File.download()`` method"""
-        if file_path is None:
-            file_path = self.name
-
+        file_path = self.name if not file_path else file_path
         if os.path.exists(file_path) and not ignore_existing:
             raise IOError('File already exists: {0}'.format(file_path))
 
@@ -461,7 +479,7 @@ class File(object):
 
         opener = urllib2.build_opener()
         opener.addheaders.append(('Cookie', cookies))
-        data = opener.open(self.download_url)
+        data = opener.open(self.url)
         with open(file_path, 'wb') as fp:
             fp.write(data.read())
 
@@ -469,15 +487,17 @@ class File(object):
     # delete()
     #_____________________________________________________________________________________
     def delete(self, debug=False, verbose=False, cascade_delete=False):
-        if cascade_delete:
-            headers = ias3.build_headers(headers={'x-archive-cascade-delete': 1})
-        else:
-            headers = ias3.build_headers()
-        endpoint = '{0}/{1}'.format(self.item.s3_endpoint, self.fname)
-        prepped_request = Request('DELETE', endpoint, headers=headers).prepare()
+        headers = s3.build_headers(cascade_delete=cascade_delete)
+        url = 'http://s3.us.archive.org/{0}/{1}'.format(self.item.identifier, self.fname)
+        request = Request(
+            method='DELETE', 
+            url=url, 
+            headers=headers,
+        )
         if debug:
-            return prepped_request 
+            return request 
         else:
             if verbose:
                 stdout.write(' deleting file: {0}\n'.format(self.name))
-            return self.item.session.send(prepped_request)
+            prepared_request = request.prepare()
+            return self.item.session.send(prepared_request)
